/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "mrlit.h"
#include "mrlog.h"

entity_t	entities[MAX_MAP_ENTITIES];
int			num_entities;

/*
============
LoadEntities
============
*/
void LoadEntities (void)
{
	char 		*data;
	entity_t	*entity;
	char		key[64];	
	epair_t		*epair;
	int			intensity;
	float		vec[3];
	int			i;

	data = dentdata;
//
// start parsing
//
	num_entities = 0;
	
// go through all the entities
	while (1)
	{
	// parse the opening brace	
		data = COM_Parse (data);
		if (!data)
			break;
		if (com_token[0] != '{')
			Error ("LoadEntities: found %s when expecting {",com_token);

		if (num_entities == MAX_MAP_ENTITIES)
			Error ("LoadEntities: MAX_MAP_ENTITIES");
		entity = &entities[num_entities];
		num_entities++;
		
	// go through all the keys in this entity
		while (1)
		{
			int		c;

		// parse key
			data = COM_Parse (data);
			if (!data)
				Error ("LoadEntities: EOF without closing brace");
			if (!strcmp(com_token,"}"))
				break;
			strcpy (key, com_token);

		// parse value
			data = COM_Parse (data);
			if (!data)
				Error ("LoadEntities: EOF without closing brace");
			c = com_token[0];
			if (c == '}')
				Error ("LoadEntities: closing brace without data");
			
			epair = malloc (sizeof(epair_t));
			memset (epair, 0, sizeof(epair));
			strcpy (epair->key, key);
			strcpy (epair->value, com_token);
			epair->next = entity->epairs;
			entity->epairs = epair;
			
			if (!strcmp(key, "classname"))
			{
				strcpy (entity->classname, com_token);
			}
			else if (!strcmp(key, "origin"))
			{
				if (sscanf(com_token, "%lf %lf %lf", &vec[0], &vec[1], &vec[2]) != 3)
				{
					Error ("LoadEntities: not 3 values for origin");
				}
				for (i=0 ; i<3 ; i++)
				{
					entity->origin[i] = vec[i];
				}
			}
			else if (!strncmp(key, "light", 5) || !strcmp (key, "_light") )
			{
				entity->light = (int)atof(com_token);
			}
			else if (!strcmp(key, "_sun0"))
			{
				vec3_t	sunangle, deviation;

				if (sscanf(com_token, "%i %f %f %f", &intensity, &vec[0], &vec[1], &vec[2]) != 4)
				{
					Error ("LoadEntities: not 4 values for _sun0");
				}

				sunlight[0] = intensity;

				sunangle[0] = vec[0];
				sunangle[1] = vec[1];
				sunangle[2] = vec[2];

				deviation[0] = 0;
				deviation[1] = 0;
				deviation[2] = 0;

				sundir[0][0][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][0][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][0][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][0]);

				deviation[0] = 5;
				deviation[1] = 0;

				sundir[0][1][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][1][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][1][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][1]);

				deviation[0] = -5;
				deviation[1] = 0;

				sundir[0][2][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][2][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][2][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][2]);

				deviation[0] = 0;
				deviation[1] = 5;

				sundir[0][3][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][3][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][3][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][3]);

				deviation[0] = 0;
				deviation[1] = -5;

				sundir[0][4][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][4][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][4][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][4]);

				deviation[0] = 5;
				deviation[1] = 5;

				sundir[0][5][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][5][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][5][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][5]);

				deviation[0] = -5;
				deviation[1] = 5;

				sundir[0][6][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][6][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][6][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][6]);

				deviation[0] = 5;
				deviation[1] = -5;

				sundir[0][7][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][7][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][7][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][7]);

				deviation[0] = -5;
				deviation[1] = -5;

				sundir[0][8][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][8][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[0][8][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][8]);
			}
			else if (!strcmp(key, "_sun1"))
			{
				vec3_t	sunangle, deviation;

				if (sscanf(com_token, "%i %f %f %f", &intensity, &vec[0], &vec[1], &vec[2]) != 4)
				{
					Error ("LoadEntities: not 4 values for _sun1");
				}

				sunlight[1] = intensity;

				sunangle[0] = vec[0];
				sunangle[1] = vec[1];
				sunangle[2] = vec[2];

				deviation[0] = 0;
				deviation[1] = 0;
				deviation[2] = 0;

				sundir[1][0][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][0][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][0][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][0]);

				deviation[0] = 5;
				deviation[1] = 0;

				sundir[1][1][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][1][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][1][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][1]);

				deviation[0] = -5;
				deviation[1] = 0;

				sundir[1][2][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][2][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][2][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][2]);

				deviation[0] = 0;
				deviation[1] = 5;

				sundir[1][3][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][3][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][3][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][3]);

				deviation[0] = 0;
				deviation[1] = -5;

				sundir[1][4][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][4][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][4][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][4]);

				deviation[0] = 5;
				deviation[1] = 5;

				sundir[1][5][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][5][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][5][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][5]);

				deviation[0] = -5;
				deviation[1] = 5;

				sundir[1][6][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][6][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][6][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][6]);

				deviation[0] = 5;
				deviation[1] = -5;

				sundir[1][7][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][7][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][7][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][7]);

				deviation[0] = -5;
				deviation[1] = -5;

				sundir[1][8][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][8][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[1][8][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][8]);
			}
			else if (!strcmp(key, "_sun2"))
			{
				vec3_t	sunangle, deviation;

				if (sscanf(com_token, "%i %f %f %f", &intensity, &vec[0], &vec[1], &vec[2]) != 4)
				{
					Error ("LoadEntities: not 4 values for _sun2");
				}

				sunlight[2] = intensity;

				sunangle[0] = vec[0];
				sunangle[1] = vec[1];
				sunangle[2] = vec[2];

				deviation[0] = 0;
				deviation[1] = 0;
				deviation[2] = 0;

				sundir[2][0][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][0][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][0][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][0]);

				deviation[0] = 5;
				deviation[1] = 0;

				sundir[2][1][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][1][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][1][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][1]);

				deviation[0] = -5;
				deviation[1] = 0;

				sundir[2][2][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][2][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][2][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][2]);

				deviation[0] = 0;
				deviation[1] = 5;

				sundir[2][3][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][3][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][3][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][3]);

				deviation[0] = 0;
				deviation[1] = -5;

				sundir[2][4][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][4][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][4][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][4]);

				deviation[0] = 5;
				deviation[1] = 5;

				sundir[2][5][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][5][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][5][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][5]);

				deviation[0] = -5;
				deviation[1] = 5;

				sundir[2][6][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][6][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][6][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][6]);

				deviation[0] = 5;
				deviation[1] = -5;

				sundir[2][7][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][7][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][7][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][7]);

				deviation[0] = -5;
				deviation[1] = -5;

				sundir[2][8][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][8][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[2][8][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][8]);
			}
			else if (!strcmp(key, "_sun3"))
			{
				vec3_t	sunangle, deviation;

				if (sscanf(com_token, "%i %f %f %f", &intensity, &vec[0], &vec[1], &vec[2]) != 4)
				{
					Error ("LoadEntities: not 4 values for _sun3");
				}

				sunlight[3] = intensity;

				sunangle[0] = vec[0];
				sunangle[1] = vec[1];
				sunangle[2] = vec[2];

				deviation[0] = 0;
				deviation[1] = 0;
				deviation[2] = 0;

				sundir[3][0][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][0][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][0][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][0]);

				deviation[0] = 5;
				deviation[1] = 0;

				sundir[3][1][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][1][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][1][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][1]);

				deviation[0] = -5;
				deviation[1] = 0;

				sundir[3][2][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][2][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][2][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][2]);

				deviation[0] = 0;
				deviation[1] = 5;

				sundir[3][3][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][3][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][3][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][3]);

				deviation[0] = 0;
				deviation[1] = -5;

				sundir[3][4][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][4][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][4][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][4]);

				deviation[0] = 5;
				deviation[1] = 5;

				sundir[3][5][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][5][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][5][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][5]);

				deviation[0] = -5;
				deviation[1] = 5;

				sundir[3][6][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][6][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][6][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][6]);

				deviation[0] = 5;
				deviation[1] = -5;

				sundir[3][7][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][7][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][7][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][7]);

				deviation[0] = -5;
				deviation[1] = -5;

				sundir[3][8][0] = (float)(-cos((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][8][1] = (float)(-sin((sunangle[1]+deviation[1])*Q_PI/180)*cos((sunangle[0]+deviation[0])*Q_PI/180));
				sundir[3][8][2] = (float)(-sin((sunangle[0]+deviation[0])*Q_PI/180));
				VectorNormalize (sundir[0][8]);
			}
		}

	// all fields have been parsed
		if (!strncmp (entity->classname, "light", 5) && !entity->light)
			entity->light = DEFAULTLIGHTLEVEL;
	}

	MRLogPrint ("%d entities read\n", num_entities);
}

char *ValueForKey (entity_t *ent, char *key)
{
	epair_t	*ep;
	
	for (ep=ent->epairs ; ep ; ep=ep->next)
		if (!strcmp (ep->key, key) )
			return ep->value;
	return "";
}

void SetKeyValue (entity_t *ent, char *key, char *value)
{
	epair_t	*ep;
	
	for (ep=ent->epairs ; ep ; ep=ep->next)
		if (!strcmp (ep->key, key) )
		{
			strcpy (ep->value, value);
			return;
		}
	ep = malloc (sizeof(*ep));
	ep->next = ent->epairs;
	ent->epairs = ep;
	strcpy (ep->key, key);
	strcpy (ep->value, value);
}

float FloatForKey (entity_t *ent, char *key)
{
	char	*k;
	
	k = ValueForKey (ent, key);
	return (float)atof(k);
}

void GetVectorForKey (entity_t *ent, char *key, vec3_t vec)
{
	char	*k;
	
	k = ValueForKey (ent, key);
	sscanf (k, "%lf %lf %lf", &vec[0], &vec[1], &vec[2]);
}

/*
================
WriteEntitiesToString
================
*/
void WriteEntitiesToString (void)
{
	char	*buf, *end;
	epair_t	*ep;
	char	line[128];
	int		i;
	
	buf = dentdata;
	end = buf;
	*end = 0;
	
	for (i=0 ; i<num_entities ; i++)
	{
		ep = entities[i].epairs;
		if (!ep)
			continue;	// ent got removed
		
		strcat (end,"{\n");
		end += 2;
				
		for (ep = entities[i].epairs ; ep ; ep=ep->next)
		{
			_snprintf (line, sizeof(line), "\"%s\" \"%s\"\n", ep->key, ep->value);
			strcat (end, line);
			end += strlen(line);
		}
		strcat (end,"}\n");
		end += 2;

		if (end > buf + MAX_MAP_ENTSTRING)
			Error ("Entity text too long");
	}
	entdatasize = end - buf + 1;
}