/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "mrlit.h"
#include "mrlog.h"

float		scaledist	= 1.0;
float		scalecos	= 0.5;
float		rangescale	= 0.5;

int			sunlight[4];
vec3_t		sundir[4][9];

byte		*filebase, *file_p, *file_end;

dmodel_t	*bspmodel;

vec3_t	bsp_origin;

qboolean	extrasamples;

byte *GetFileSpace (int size)
{
	byte	*buf;
	
	file_p = (byte *)(((long)file_p + 3)&~3);
	buf = file_p;
	file_p += size;

	if (file_p > file_end)
		Error ("GetFileSpace: overrun");
	return buf;
}

void LightThread (int num)
{
	int	i = 0;
	
	while (1)
	{
		i++;

		if (i >= numfaces)
			return;
		
		LightFace (i, num);
	}
}

/*
=============
LightWorld
=============
*/
void LightWorld (int num)
{
	filebase = file_p = dlightdata[num];
	file_end = filebase + MAX_MAP_LIGHTING;

	LightThread (num);

	lightdatasize[num] = file_p - filebase;
	
	MRLogPrint ("lightdatasize[%i]: %i\n", num, lightdatasize[num]);
}

/*
========
main

light modelfile
========
*/
int main (int argc, char **argv)
{
	int		i;
	double		start, end;
	char		sourcename[1024];
	char		logname[1024];

	for (i=1 ; i<argc ; i++)
	{
		if (!strcmp(argv[i],"-extra"))
		{
			extrasamples = true;
			MRLogPrint ("extra sampling enabled\n");
		}
		else if (!strcmp(argv[i],"-dist"))
		{
			scaledist = (float)atof (argv[i+1]);
			i++;
		}
		else if (!strcmp(argv[i],"-range"))
		{
			rangescale = (float)atof (argv[i+1]);
			i++;
		}
		else if (argv[i][0] == '-')
			Error ("Unknown option \"%s\"", argv[i]);
		else
			break;
	}

	if (i != argc - 1)
		Error ("usage: mrlit [options] [sourcefile]\noptions: -extra");

	start = I_FloatTime ();

	strcpy (sourcename, argv[i]);
	StripExtension (sourcename);
	strcat (sourcename, ".bsp");

	strcpy (logname, sourcename);
	StripExtension (logname);
	strcat (logname, "_lit.log");

	Init_MRLog (logname);

    MRLogPrint ("\n----- MRLIT -----\n\n");
	MRLogPrint ("inputfile:  %s\n", sourcename);
	MRLogPrint ("logfile:    %s\n", logname);

	LoadBSPFile (sourcename);
	LoadEntities ();
		
	MakeTnodes (&dmodels[0]);

	LightWorld (0);
	LightWorld (1);
	LightWorld (2);
	LightWorld (3);

	WriteEntitiesToString ();	
	WriteBSPFile (sourcename);

	end = I_FloatTime ();
	MRLogPrint ("%5.1f seconds elapsed\n", end-start);
	
	return 0;
}

