/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __MATHLIB__
#define __MATHLIB__

typedef float vec3_t[3];

float	DotProduct (float v1[3], float v2[3]); 

#define	SIDE_FRONT		0
#define	SIDE_ON			2
#define	SIDE_BACK		1
#define	SIDE_CROSS		-2

#define	Q_PI	3.14159265358979323846

extern vec3_t vec3_origin;

#define	EQUAL_EPSILON	0.001

#define VectorLength(v) (sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]))
#define VectorCompare(a,b) ((a[0] == b[0]) & (a[1] == b[1]) & (a[2] == b[2]))

#define VectorSubtract(a,b,c)					\
{												\
	c[0]	= a[0] - b[0];						\
	c[1]	= a[1] - b[1];						\
	c[2]	= a[2] - b[2];						\
}

#define VectorAdd(a,b,c)						\
{												\
	c[0]	= a[0] + b[0];						\
	c[1]	= a[1] + b[1];						\
	c[2]	= a[2] + b[2];						\
}

#define VectorCopy(in,out)						\
{												\
	out[0]	= in[0];							\
	out[1]	= in[1];							\
	out[2]	= in[2];							\
}

#define VectorMA(a,s,b,c)						\
{												\
	c[0]	= a[0] + (s * b[0]);				\
	c[1]	= a[1] + (s * b[1]);				\
	c[2]	= a[2] + (s * b[2]);				\
}

#define CrossProduct(v1,v2,cross)				\
{												\
	cross[0] = v1[1] * v2[2] - v1[2] * v2[1];	\
	cross[1] = v1[2] * v2[0] - v1[0] * v2[2];	\
	cross[2] = v1[0] * v2[1] - v1[1] * v2[0];	\
}

#define VectorInverse(v)						\
{												\
	v[0]	= -v[0];							\
	v[1]	= -v[1];							\
	v[2]	= -v[2];							\
}

#define VectorScale(a,b,c)						\
{												\
	c[0]	= a[0] * b;							\
	c[1]	= a[1] * b;							\
	c[2]	= a[2] * b;							\
}

void VectorNormalize (vec3_t v);

#endif // __MATHLIB__